@@ -23,6 +23,7 @@ from textparser import Optional
 from ..attribute_definition import AttributeDefinition
 from ..attribute import Attribute
 from ..signal import Signal
+from ..signal import NamedSignalValue
 from ..signal import Decimal as SignalDecimal
 from ..signal_group import SignalGroup
 from ..message import Message
@@ -80,13 +81,13 @@ DBC_FMT = (
     '\r\n'
     '\r\n'
     '{cm}\r\n'
-    '{signal_types}'
     '{ba_def}\r\n'
     '{ba_def_def}\r\n'
     '{ba}\r\n'
     '{val}\r\n'
+    '{signal_types}\r\n'
     '{sig_group}\r\n'
-    '\r\n'
+    '{sig_mux_values}\r\n'
 )
 
 
@@ -183,7 +184,7 @@ class Parser(textparser.Parser):
 
         token_specs = [
             ('SKIP',     r'[ \r\n\t]+|//.*?\n'),
-            ('NUMBER',   r'-?\d+\.?\d*([eE][+-]?\d+)?'),
+            ('NUMBER',   r'[-+]?\d+\.?\d*([eE][+-]?\d+)?'),
             ('WORD',     r'[A-Za-z0-9_]+'),
             ('STRING',   r'"(\\"|[^"])*?"'),
             ('LPAREN',   r'\('),
@@ -319,7 +320,7 @@ class Parser(textparser.Parser):
             'BO_TX_BU_', 'NUMBER', ':', DelimitedList('WORD'), ';')
 
         signal_group = Sequence(
-            'SIG_GROUP_', 'NUMBER', 'WORD', 'NUMBER', ':', OneOrMore('WORD'), ';')
+            'SIG_GROUP_', 'NUMBER', 'WORD', 'NUMBER', ':', ZeroOrMore('WORD'), ';')
 
         return OneOrMoreDict(
             choice(
@@ -558,6 +559,12 @@ def _dump_senders(database):
 def _dump_comments(database):
     cm = []
 
+    for bus in database.buses:
+        if bus.comment is not None:
+            cm.append(
+                'CM_ "{comment}";'.format(
+                    comment=bus.comment) )
+
     for node in database.nodes:
         if node.comment is not None:
             cm.append(
@@ -593,20 +600,33 @@ def _dump_signal_types(database):
 
             valtype.append(
                 'SIG_VALTYPE_ {} {} : {};'.format(
-                    message.frame_id,
+                    get_dbc_frame_id(message),
                     signal.name,
                     FLOAT_LENGTH_TO_SIGNAL_TYPE[signal.length]))
 
     return valtype
 
+def _create_GenMsgCycleTime_definition():
+    return AttributeDefinition('GenMsgCycleTime',
+                               default_value=0,
+                               kind='BO_',
+                               type_name='INT',
+                               minimum=0,
+                               maximum=2**16-1)
 
 def _dump_attribute_definitions(database):
     ba_def = []
 
     if database.dbc is None:
-        return ba_def
+        definitions = odict()
+    else:
+        definitions = database.dbc.attribute_definitions
 
-    definitions = database.dbc.attribute_definitions
+    # define "GenMsgCycleTime" attribute for specifying the cycle
+    # times of messages if it has not been explicitly defined
+    if 'GenMsgCycleTime' not in definitions:
+        definitions['GenMsgCycleTime'] = \
+            _create_GenMsgCycleTime_definition()
 
     def get_value(definition, value):
         if definition.minimum is None:
@@ -657,9 +677,15 @@ def _dump_attribute_definition_defaults(database):
     ba_def_def = []
 
     if database.dbc is None:
-        return ba_def_def
+        definitions = odict()
+    else:
+        definitions = database.dbc.attribute_definitions
 
-    definitions = database.dbc.attribute_definitions
+    # define "GenMsgCycleTime" attribute for specifying the cycle
+    # times of messages if it has not been explicitly defined
+    if 'GenMsgCycleTime' not in definitions:
+        definitions['GenMsgCycleTime'] = \
+            _create_GenMsgCycleTime_definition()
 
     for definition in definitions.values():
         if definition.default_value is not None:
@@ -673,7 +699,6 @@ def _dump_attribute_definition_defaults(database):
 
     return ba_def_def
 
-
 def _dump_attributes(database):
     ba = []
 
@@ -688,43 +713,49 @@ def _dump_attributes(database):
     if database.dbc is not None:
         if database.dbc.attributes is not None:
             for attribute in database.dbc.attributes.values():
-                ba.append(
-                    'BA_ "{name}" {value};'.format(name=attribute.definition.name,
-                                                   value=get_value(attribute)))
+                ba.append(f'BA_ "{attribute.definition.name}" '
+                          f'{get_value(attribute)};')
 
     for node in database.nodes:
         if node.dbc is not None:
             if node.dbc.attributes is not None:
                 for attribute in node.dbc.attributes.values():
-                    ba.append(
-                        'BA_ "{name}" {kind} {node_name} {value};'.format(
-                            name=attribute.definition.name,
-                            kind=attribute.definition.kind,
-                            node_name=node.name,
-                            value=get_value(attribute)))
+                    ba.append(f'BA_ "{attribute.definition.name}" '
+                              f'{attribute.definition.kind} '
+                              f'{node.name} '
+                              f'{get_value(attribute)};')
 
     for message in database.messages:
-        if message.dbc is not None:
-            if message.dbc.attributes is not None:
-                for attribute in message.dbc.attributes.values():
-                    ba.append(
-                        'BA_ "{name}" {kind} {frame_id} {value};'.format(
-                            name=attribute.definition.name,
-                            kind=attribute.definition.kind,
-                            frame_id=get_dbc_frame_id(message),
-                            value=get_value(attribute)))
+        # retrieve the ordered dictionary of message attributes
+        msg_attributes = odict()
+        if message.dbc is not None and message.dbc.attributes is not None:
+            msg_attributes = message.dbc.attributes
 
+        # synchronize the attribute for the message cycle time with
+        # the cycle time specified by the message object
+        if message.cycle_time is None and 'GenMsgCycleTime' in msg_attributes:
+            del msg_attributes['GenMsgCycleTime']
+        elif message.cycle_time is not None:
+            msg_attributes['GenMsgCycleTime'] = \
+                Attribute(value=message.cycle_time,
+                          definition=_create_GenMsgCycleTime_definition())
+
+        # output all message attributes
+        for attribute in msg_attributes.values():
+            ba.append(f'BA_ "{attribute.definition.name}" '
+                      f'{attribute.definition.kind} '
+                      f'{get_dbc_frame_id(message)} '
+                      f'{get_value(attribute)};')
+
+        # handle the signals contained in the message
         for signal in message.signals[::-1]:
-            if signal.dbc is not None:
-                if signal.dbc.attributes is not None:
-                    for attribute in signal.dbc.attributes.values():
-                        ba.append(
-                            'BA_ "{name}" {kind} {frame_id} {signal_name} {value};'.format(
-                                name=attribute.definition.name,
-                                kind=attribute.definition.kind,
-                                frame_id=get_dbc_frame_id(message),
-                                signal_name=signal.name,
-                                value=get_value(attribute)))
+            if signal.dbc is not None and signal.dbc.attributes is not None:
+                for attribute in signal.dbc.attributes.values():
+                    ba.append(f'BA_ "{attribute.definition.name}" '
+                              f'{attribute.definition.kind} '
+                              f'{get_dbc_frame_id(message)} '
+                              f'{signal.name} '
+                              f'{get_value(attribute)};')
 
     return ba
 
@@ -769,11 +800,92 @@ def _dump_signal_groups(database):
     return sig_group
 
 
+def _is_extended_mux_needed(messages):
+    """Check for messages with more than one mux signal or signals with
+    more than one multiplexer value.
+
+    """
+
+    for message in messages:
+        multiplexers = [
+            signal.name
+            for signal in message.signals
+            if signal.is_multiplexer
+        ]
+
+        if len(multiplexers) > 1:
+            return True
+
+        for signal in message.signals:
+            if signal.multiplexer_ids:
+                if len(signal.multiplexer_ids) > 1:
+                    return True
+
+    return False
+
+
+def _create_mux_ranges(multiplexer_ids):
+    """Create a list of ranges based on a list of single values.
+
+    Example:
+        Input:  [1, 2, 3, 5,      7, 8, 9]
+        Output: [[1, 3], [5, 5], [7, 9]]
+
+    """
+
+    ordered = sorted(multiplexer_ids)
+    # Anything but ordered[0] - 1
+    prev_value = ordered[0]
+    ranges = []
+
+    for value in ordered:
+        if value == prev_value + 1:
+            ranges[-1][1] = value
+        else:
+            ranges.append([value, value])
+
+        prev_value = value
+
+    return ranges
+
+
+def _dump_signal_mux_values(database):
+    """Create multiplex entries ("SG_MUL_VAL_") if extended multiplexing
+    is used.
+
+    """
+
+    if not _is_extended_mux_needed(database.messages):
+        return []
+
+    sig_mux_values = []
+
+    for message in database.messages:
+        for signal in message.signals:
+            if not signal.multiplexer_ids:
+                continue
+
+            ranges = ', '.join([
+                '{}-{}'.format(minimum, maximum)
+                for minimum, maximum in _create_mux_ranges(signal.multiplexer_ids)
+            ])
+
+            sig_mux_values.append(
+                'SG_MUL_VAL_ {frame_id} {name} {multiplexer} {ranges};'.format(
+                    frame_id=get_dbc_frame_id(message),
+                    name=signal.name,
+                    multiplexer=signal.multiplexer_signal,
+                    ranges=ranges))
+
+    return sig_mux_values
+
+
 def _load_comments(tokens):
     comments = defaultdict(dict)
 
     for comment in tokens.get('CM_', []):
         if not isinstance(comment[1], list):
+            comments['database']['bus'] = comment[1]
             continue
 
         item = comment[1]
@@ -894,7 +1006,8 @@ def _load_value_tables(tokens):
 
     for value_table in tokens.get('VAL_TABLE_', []):
         name = value_table[1]
-        choices = {int(number): text for number, text in value_table[2]}
+        choices = {int(number): NamedSignalValue(number, text) for number, text in value_table[2]}
+        #choices = {int(number): text for number, text in value_table[2]}
         value_tables[name] = choices
 
     return value_tables
@@ -926,7 +1039,7 @@ def _load_choices(tokens):
         if len(choice[1]) == 0:
             continue
 
-        od = odict((int(''.join(v[0])), v[1]) for v in choice[3])
+        od = odict((int(v[0]), NamedSignalValue(v[0], v[1])) for v in choice[3])
 
         if len(od) == 0:
             continue
@@ -1156,6 +1269,14 @@ def _load_signals(tokens,
         except (KeyError, TypeError):
             return None
 
+    def get_signal_spn(frame_id_dbc, name):
+        signal_attributes = get_attributes(frame_id_dbc, name)
+
+        try:
+            return signal_attributes['SPN'].value
+        except (KeyError, TypeError):
+            return None
+
     signals = []
 
     for signal in tokens:
@@ -1180,6 +1301,7 @@ def _load_signals(tokens,
                                          get_maximum_decimal(signal[15],
                                                              signal[17])),
                    unit=(None if signal[19] == '' else signal[19]),
+                   spn=get_signal_spn(frame_id_dbc, signal[1][0]),
                    choices=get_choices(frame_id_dbc,
                                        signal[1][0]),
                    dbc_specifics=DbcSpecifics(get_attributes(frame_id_dbc, signal[1][0]),
@@ -1259,10 +1381,12 @@ def _load_messages(tokens,
         message_attributes = get_attributes(frame_id_dbc)
 
         try:
-            return int(message_attributes['GenMsgCycleTime'].value)
+            result = int(message_attributes['GenMsgCycleTime'].value)
+            return None if result == 0 else result
         except (KeyError, TypeError):
             try:
-                return int(definitions['GenMsgCycleTime'].default_value)
+                result = int(definitions['GenMsgCycleTime'].default_value)
+                return None if result == 0 else result
             except (KeyError, TypeError):
                 return None
 
@@ -1371,7 +1495,7 @@ def _load_version(tokens):
     return tokens.get('VERSION', [[None, None]])[0][1]
 
 
-def _load_bus(attributes):
+def _load_bus(attributes, comments):
     try:
         bus_name = attributes['database']['DBName'].value
     except KeyError:
@@ -1382,7 +1506,12 @@ def _load_bus(attributes):
     except KeyError:
         bus_baudrate = None
 
-    return Bus(bus_name, baudrate=bus_baudrate)
+    try:
+        bus_comment = comments['database']['bus']
+    except KeyError:
+        bus_comment = None
+
+    return Bus(bus_name, baudrate=bus_baudrate, comment=bus_comment)
 
 
 def _load_nodes(tokens, comments, attributes, definitions):
@@ -1535,6 +1664,7 @@ def dump_string(database):
     ba = _dump_attributes(database)
     val = _dump_choices(database)
     sig_group = _dump_signal_groups(database)
+    sig_mux_values = _dump_signal_mux_values(database)
 
     return DBC_FMT.format(version=_dump_version(database),
                           bu=' '.join(bu),
@@ -1547,7 +1677,8 @@ def dump_string(database):
                           ba_def_def='\r\n'.join(ba_def_def),
                           ba='\r\n'.join(ba),
                           val='\r\n'.join(val),
-                          sig_group='\r\n'.join(sig_group))
+                          sig_group='\r\n'.join(sig_group),
+                          sig_mux_values='\r\n'.join(sig_mux_values))
 
 
 def get_definitions_dict(definitions, defaults):
@@ -1602,7 +1733,7 @@ def load_string(string, strict=True):
     defaults = _load_attribute_definition_defaults(tokens)
     attribute_definitions = get_definitions_dict(definitions, defaults)
     attributes = _load_attributes(tokens, attribute_definitions)
-    bus = _load_bus(attributes)
+    bus = _load_bus(attributes, comments)
     value_tables = _load_value_tables(tokens)
     choices = _load_choices(tokens)
     message_senders = _load_message_senders(tokens, attributes)
