@@ -1,4 +1,4 @@
-from __future__ import print_function
+import argparse
 
 from .. import database
 
@@ -6,6 +6,7 @@ from .. import database
 def _do_convert(args):
     dbase = database.load_file(args.infile,
                                encoding=args.encoding,
+                               prune_choices=args.prune,
                                strict=not args.no_strict)
 
     database.dump_file(dbase,
@@ -17,10 +18,19 @@ def _do_convert(args):
 def add_subparser(subparsers):
     convert_parser = subparsers.add_parser(
         'convert',
-        description='Convert given database from one format to another.')
+        description='Convert given database from one format to another.',
+        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
     convert_parser.add_argument(
         '-e', '--encoding',
         help='File encoding.')
+    # the result of the convert operation should represent the
+    # original file as closely as possible so -- in contrast to the
+    # other subparsers -- we do not prune the names of signal values
+    # by default
+    convert_parser.add_argument(
+        '--prune',
+        action='store_true',
+        help='Shorten the named signal values.')
     convert_parser.add_argument(
         '--no-strict',
         action='store_true',
